#!/bin/sh

usage() {
	echo "$0 <testname>"
}

if [ $# -ne 1 ]; then
	usage
	exit 1
fi

if [ ! -f config.mak ]; then
	echo "run ./configure first."
	exit 1
fi
. config.mak

TESTNAME=$1

RAMSIZE=256M
TESTFLAT=$TEST_DIR/$TESTNAME.flat
VMNAME=$TESTNAME

if [ ! -f $SRCDIR/$TESTFLAT ]; then
	echo Couldn\'t find \'$TESTFLAT\'\; ensure \"$TESTNAME\" is a valid \
	     test and run gmake first.
	exit 1
fi

GRUBCFG=`mktemp -q .grubbhyvecfg.XXXXXX`
if [ $? -ne 0 ]; then
	echo "Failed to create temp file."
	exit 1
fi
cat > $GRUBCFG <<- EOF
	set timeout=0
	menuentry '$TESTNAME' {
		multiboot \$prefix/$TESTFLAT
	}
EOF
sudo grub-bhyve \
	--directory="$SRCDIR" \
	--grub-cfg=$GRUBCFG \
	--memory=$RAMSIZE \
	--root=host \
	--verbose \
	$VMNAME \
	> /dev/null 2>&1
rm $GRUBCFG

sudo bhyve \
	-c 1 \
	-s 0,hostbridge \
	-s 3,lpc \
	-l com1,stdio \
	-l pc-testdev \
	-H \
	-m $RAMSIZE \
	$VMNAME

BHYVE_EXIT=$?

echo -n "bhyve exited: "
if [ $(($BHYVE_EXIT & 1)) -eq 1 ]; then
	# Assume this is an exit code from the 'debugexit' port:
	CODE=$(($BHYVE_EXIT >> 1))
	if [ $CODE -eq 63 ]; then
		# (See 'lib/abort.c').
		echo "unit test abort!"
	else
		echo "unit test exit: $CODE"
	fi
else
	echo "non-'debugexit' exit status: $BHYVE_EXIT"
fi

sudo bhyvectl --vm=$VMNAME --destroy > /dev/null 2>&1

exit $BHYVE_EXIT
